@@ -9,7 +9,7 @@ URLBASE = "%s/%s/user" % (Fitbit.API_ENDPOINT, Fitbit.API_VERSION)
 
 class TestBase(TestCase):
     def setUp(self):
-        self.fb = Fitbit(consumer_key='x', consumer_secret='y')
+        self.fb = Fitbit('x', 'y')
 
     def common_api_test(self, funcname, args, kwargs, expected_args, expected_kwargs):
         # Create a fitbit object, call the named function on it with the given
@@ -33,7 +33,7 @@ class APITest(TestBase):
         KWARGS = { 'a': 3, 'b': 4, 'headers': {'Accept-Language': self.fb.SYSTEM}}
         mock_response = mock.Mock()
         mock_response.status_code = 200
-        mock_response.content = "1"
+        mock_response.content = b"1"
         with mock.patch.object(self.fb.client, 'make_request') as client_make_request:
             client_make_request.return_value = mock_response
             retval = self.fb.make_request(*ARGS, **KWARGS)
@@ -155,7 +155,7 @@ class CollectionResourceTest(TestBase):
         # since the __init__ is going to set up references to it
         with mock.patch('fitbit.api.Fitbit._COLLECTION_RESOURCE') as coll_resource:
             coll_resource.return_value = 999
-            fb = Fitbit(consumer_key='x', consumer_secret='y')
+            fb = Fitbit('x', 'y')
             retval = fb.body(date=1, user_id=2, data=3)
         args, kwargs = coll_resource.call_args
         self.assertEqual(('body',), args)
@@ -181,7 +181,7 @@ class DeleteCollectionResourceTest(TestBase):
         # since the __init__ is going to set up references to it
         with mock.patch('fitbit.api.Fitbit._DELETE_COLLECTION_RESOURCE') as delete_resource:
             delete_resource.return_value = 999
-            fb = Fitbit(consumer_key='x', consumer_secret='y')
+            fb = Fitbit('x', 'y')
             retval = fb.delete_water(log_id=log_id)
         args, kwargs = delete_resource.call_args
         self.assertEqual(('water',), args)
@@ -193,7 +193,7 @@ class MiscTest(TestBase):
     def test_recent_activities(self):
         user_id = "LukeSkywalker"
         with mock.patch('fitbit.api.Fitbit.activity_stats') as act_stats:
-            fb = Fitbit(consumer_key='x', consumer_secret='y')
+            fb = Fitbit('x', 'y')
             retval = fb.recent_activities(user_id=user_id)
         args, kwargs = act_stats.call_args
         self.assertEqual((), args)
@@ -287,9 +287,55 @@ class MiscTest(TestBase):
     def test_activities(self):
         url = "%s/%s/activities.json" % (Fitbit.API_ENDPOINT, Fitbit.API_VERSION)
         self.common_api_test('activities_list', (), {}, (url,), {})
+        url = "%s/%s/user/-/activities.json" % (Fitbit.API_ENDPOINT, Fitbit.API_VERSION)
+        self.common_api_test('log_activity', (), {'data' : 'FOO'}, (url,), {'data' : 'FOO'} )
         url = "%s/%s/activities/FOOBAR.json" % (Fitbit.API_ENDPOINT, Fitbit.API_VERSION)
         self.common_api_test('activity_detail', ("FOOBAR",), {}, (url,), {})
 
+    def test_bodyweight(self):
+        def test_get_bodyweight(fb, base_date=None, user_id=None, period=None, end_date=None, expected_url=None):
+            with mock.patch.object(fb, 'make_request') as make_request:
+                fb.get_bodyweight(base_date, user_id=user_id, period=period, end_date=end_date)
+            args, kwargs = make_request.call_args
+            self.assertEqual((expected_url,), args)
+
+        user_id = 'BAR'
+
+        # No end_date or period
+        test_get_bodyweight(self.fb, base_date=datetime.date(1992, 5, 12), user_id=None, period=None, end_date=None,
+            expected_url=URLBASE + "/-/body/log/weight/date/1992-05-12.json")
+        # With end_date
+        test_get_bodyweight(self.fb, base_date=datetime.date(1992, 5, 12), user_id=user_id, period=None, end_date=datetime.date(1998, 12, 31),
+            expected_url=URLBASE + "/BAR/body/log/weight/date/1992-05-12/1998-12-31.json")
+        # With period
+        test_get_bodyweight(self.fb, base_date=datetime.date(1992, 5, 12), user_id=user_id, period="1d", end_date=None,
+            expected_url=URLBASE + "/BAR/body/log/weight/date/1992-05-12/1d.json")
+        # Date defaults to today
+        test_get_bodyweight(self.fb, base_date=None, user_id=None, period=None, end_date=None,
+            expected_url=URLBASE + "/-/body/log/weight/date/%s.json" % datetime.date.today().strftime('%Y-%m-%d'))
+
+    def test_bodyfat(self):
+        def test_get_bodyfat(fb, base_date=None, user_id=None, period=None, end_date=None, expected_url=None):
+            with mock.patch.object(fb, 'make_request') as make_request:
+                fb.get_bodyfat(base_date, user_id=user_id, period=period, end_date=end_date)
+            args, kwargs = make_request.call_args
+            self.assertEqual((expected_url,), args)
+
+        user_id = 'BAR'
+
+        # No end_date or period
+        test_get_bodyfat(self.fb, base_date=datetime.date(1992, 5, 12), user_id=None, period=None, end_date=None,
+            expected_url=URLBASE + "/-/body/log/fat/date/1992-05-12.json")
+        # With end_date
+        test_get_bodyfat(self.fb, base_date=datetime.date(1992, 5, 12), user_id=user_id, period=None, end_date=datetime.date(1998, 12, 31),
+            expected_url=URLBASE + "/BAR/body/log/fat/date/1992-05-12/1998-12-31.json")
+        # With period
+        test_get_bodyfat(self.fb, base_date=datetime.date(1992, 5, 12), user_id=user_id, period="1d", end_date=None,
+            expected_url=URLBASE + "/BAR/body/log/fat/date/1992-05-12/1d.json")
+        # Date defaults to today
+        test_get_bodyfat(self.fb, base_date=None, user_id=None, period=None, end_date=None,
+            expected_url=URLBASE + "/-/body/log/fat/date/%s.json" % datetime.date.today().strftime('%Y-%m-%d'))
+
     def test_friends(self):
         url = URLBASE + "/-/friends.json"
         self.common_api_test('get_friends', (), {}, (url,), {})
@@ -326,3 +372,71 @@ class MiscTest(TestBase):
         url = URLBASE + "/-/COLLECTION/apiSubscriptions/SUBSCRIPTION_ID-COLLECTION.json"
         self.common_api_test('subscription', ("SUBSCRIPTION_ID", "SUBSCRIBER_ID"), {'method': 'THROW', 'collection': "COLLECTION"},
             (url,), {'method': 'THROW', 'headers': {'X-Fitbit-Subscriber-id': "SUBSCRIBER_ID"}})
+
+    def test_alarms(self):
+        url = "%s/-/devices/tracker/%s/alarms.json" % (URLBASE, 'FOO')
+        self.common_api_test('get_alarms', (), {'device_id': 'FOO'}, (url,), {})
+        url = "%s/-/devices/tracker/%s/alarms/%s.json" % (URLBASE, 'FOO', 'BAR')
+        self.common_api_test('delete_alarm', (), {'device_id': 'FOO', 'alarm_id': 'BAR'}, (url,), {'method': 'DELETE'})
+        url = "%s/-/devices/tracker/%s/alarms.json" % (URLBASE, 'FOO')
+        self.common_api_test('add_alarm',
+            (),
+            {'device_id': 'FOO',
+             'alarm_time': datetime.datetime(year=2013, month=11, day=13, hour=8, minute=16),
+             'week_days': ['MONDAY']
+            },
+            (url,),
+            {'data':
+                 {'enabled': True,
+                    'recurring': False,
+                    'time': datetime.datetime(year=2013, month=11, day=13, hour=8, minute=16).strftime("%H:%M%z"),
+                    'vibe': 'DEFAULT',
+                    'weekDays': ['MONDAY'],
+                },
+            'method': 'POST'
+            }
+        )
+        self.common_api_test('add_alarm',
+            (),
+            {'device_id': 'FOO',
+             'alarm_time': datetime.datetime(year=2013, month=11, day=13, hour=8, minute=16),
+             'week_days': ['MONDAY'], 'recurring': True, 'enabled': False, 'label': 'ugh',
+             'snooze_length': 5,
+             'snooze_count': 5
+            },
+            (url,),
+            {'data':
+                 {'enabled': False,
+                  'recurring': True,
+                  'label': 'ugh',
+                  'snoozeLength': 5,
+                  'snoozeCount': 5,
+                  'time': datetime.datetime(year=2013, month=11, day=13, hour=8, minute=16).strftime("%H:%M%z"),
+                  'vibe': 'DEFAULT',
+                  'weekDays': ['MONDAY'],
+                },
+            'method': 'POST'}
+        )
+        url = "%s/-/devices/tracker/%s/alarms/%s.json" % (URLBASE, 'FOO', 'BAR')
+        self.common_api_test('update_alarm',
+            (),
+            {'device_id': 'FOO',
+             'alarm_id': 'BAR',
+             'alarm_time': datetime.datetime(year=2013, month=11, day=13, hour=8, minute=16),
+             'week_days': ['MONDAY'], 'recurring': True, 'enabled': False, 'label': 'ugh',
+             'snooze_length': 5,
+             'snooze_count': 5
+            },
+            (url,),
+            {'data':
+                 {'enabled': False,
+                  'recurring': True,
+                  'label': 'ugh',
+                  'snoozeLength': 5,
+                  'snoozeCount': 5,
+                  'time': datetime.datetime(year=2013, month=11, day=13, hour=8, minute=16).strftime("%H:%M%z"),
+                  'vibe': 'DEFAULT',
+                  'weekDays': ['MONDAY'],
+                },
+            'method': 'POST'}
+        )
